from pyoneall import OneAll
from django.contrib.auth.models import User
from django.conf import settings
from django.db.models import get_model

OneAllUserIdentity = get_model('django_oneall', 'OneAllUserIdentity')

# The worker to be used for authentication
oneall = OneAll(settings.ONEALL_SITE_NAME, settings.ONEALL_PUBLIC_KEY, settings.ONEALL_PRIVATE_KEY)

class OneAllAuthBackend(object):
    """
    OneAll Authentication Backend.
    """
    def authenticate(self, token):
        """
        Performs authentication using a connection token. Creates and updates User and OneAllUserIdentity
        if necessary.
        :param str token: OneAll connection token
        """
        oa_user = oneall.connection(token).user

        # Check if user exists and create one if not
        identity, created = OneAllUserIdentity.objects.get_or_create(
            user_token=oa_user.user_token,
            raw=unicode(oa_user.identity)
        )

        # Update cache for new users and for existing users if setting is provided
        if created:
            identity.update_user_cache()
        elif getattr(settings, 'ONEALL_REFRESH_CACHE_ON_AUTH', True):
            identity.refresh(raw=oa_user.identity)
            identity.update_user_cache()

        # Return authenticated user
        return identity.user

    def get_user(self, user_id):
        """
        Retrieve user by user ID
        :param user_id: User ID
        """
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None



from django.contrib.auth.models import User
from django.db import models
from django.conf import settings
from pyoneall.base import OADict

class OneAllUserIdentity(models.Model):
    """
    OneAll User Identity Model
    Caches raw JSON corresponding with user's social identity allow instant retrieval of user details.
    """
    user_token = models.CharField(max_length=36, primary_key=True)
    raw = models.CharField(max_length=8192, default='{}')
    user = models.OneToOneField(User, related_name="identity", null=True)

    def __init__(self, *args, **kwargs):
        """
        Upon creation, creates attributes to correspond with the cached data in `raw`
        """
        super(self.__class__, self).__init__(*args, **kwargs)
        self.__dict__.update(OADict(**eval(self.raw)))

    def refresh(self, raw=None):
        """
        Refresh identity cache from OneAll
        """
        if not raw:
            from auth import oneall
            raw = oneall.user(self.user.username).identities.identity[0]
        raw.pop('id', None)
        raw.pop('user', None)
        self.raw = unicode(raw)
        self.__dict__.update(OADict(**eval(self.raw)))
        self.save()

    def update_user_cache(self):
        """
        Update selected fields in the User model from social identity
        """
        user = self.user if self.user else User()
        for field, values in getattr(settings, 'ONEALL_CACHE_FIELDS', {}).iteritems():
            for value in values:
                try:
                    setattr(user, field, eval('self.%s' % value))
                    print eval('self.%s' % value)
                except Exception as e:
                    print e
        user.save()
        if not self.user:
            self.user = user
            self.save()

    class Meta:
        db_table = getattr(settings, 'ONEALL_CACHE_TABLE', 'oneall_cache')



import os
from setuptools import setup
README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
setup(name='django-oneall',
    version='0.1.2',
    packages=['django_oneall'],
    install_requires=['Django >= 1.4', 'pyoneall == 0.1'],
    license='MIT License, see LICENSE file',
    description='Django Authentication support for OneAll. Provides unified authentication for 20+ social networks',
    long_description=README,
    url='http://www.leandigo.com/',
    author='Michael Greenberg / Leandigo',
    author_email='michael@leandigo.com'
)



from django.views.decorators.csrf import csrf_exempt
from django.contrib.auth import authenticate, login
from django.conf import settings
from django.http import HttpResponseRedirect

@csrf_exempt
def oneall_auth(request):
    """
    Callback view for OneAll Authentication
    :returns HttpResponseRedirect: A redirection to the LOGIN_REDIRECT_URL
    """
    connection_token=request.POST['connection_token']
    user = authenticate(token=connection_token)
    login(request, user)
    return HttpResponseRedirect(settings.LOGIN_REDIRECT_URL)
